package cd.casic.ci.common.pipeline.container;

import cd.casic.ci.common.pipeline.pojo.agent.AgentType;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.container
 * @Project：ops-pro
 * @name：AgentReuseMutex
 * @Date：2025/03/25 16:51
 * @Filename：AgentReuseMutex
 * @description： * AgentReuseMutex 构建机复用互斥
 * * 在同一次构建中总共有五种情况
 * * 1、单个Agent节点且明确指定了使用某个Agent(AgentId或者AgentName)，这种情况在item中可以直接拿到，直接进行项目级的互斥逻辑
 * * 2、单个Agent节点但使用变量指代Agent
 * *   2-1、被指代的Agent节点与复用他的Job处在同一Stage下，并行执行，这时被指代的Agent与复用他的Job进行同样的变量替换，
 * *   并行竞争，没有先后顺序
 * *   2-2、被指代的Agent节点与复用他的Job处在同一Stage下，串行执行(即流水线的Job依赖)，这时被指代的Agent应先于复用他的Job执行，
 * *   且在Var表中写入可以拿到Agent的只读变量，复用Job读取变量执行
 * * 3、Agent集群且明确指定了某个集群(集群ID或者名称)，在分发到Dispatch前需要进行集群级别的互斥锁逻辑，
 * * 这样可以保证同样进行互斥逻辑的集群在引擎处进行等待。
 * * 与单个Agent节点的Job关系相同，并行执行时并行竞争，串行时天然的有先后顺序，读取Var表中节点进行项目级的互斥逻辑
 * * 4、Agent集群但使用变量指代，与未进行变量指代的逻辑一致，无需特殊声明
 * * 注：如果是共享集群不受影响，依旧使用执行项目来进行锁定
 * * @param reUseJobId 如果为空说明当前job为被依赖job
 * * @param agentOrEnvId agentId/agentName 或者 envId/envName 或者变量，根据 type 区分不同值
 * * @param type 复用互斥的各种类型
 * * @param endJob 是否是最后一波的Job
 */
@Data
@NoArgsConstructor
@Schema(title = "构建机复用互斥模型")
public class AgentReuseMutex {
    private String jobId;
    private String reUseJobId;
    private String agentOrEnvId;
    private AgentReuseMutexType type;
    private Boolean endJob;
    @Schema(title = "是否排队", required = false)
    private Boolean queueEnable = true;
    @Schema(title = "排队等待时间（分钟）0表示不等待直接失败", required = false)
    private Integer timeout = 900;
    @Schema(title = "支持变量解析的timeout，变量值非数字则会改取timeout值", required = false)
    private String timeoutVar;
    @Schema(title = "排队队列大小", required = false)
    private Integer queue = 10;
    @Schema(title = "运行时的agentOrEnvId，如果有值说明已经初始化了", required = false)
    private String runtimeAgentOrEnvId;
    @Schema(title = "占用锁定的信息用于日志提示", required = false)
    private String linkTip;

    public static class Companion {
        public static final long AGENT_LOCK_TIMEOUT = 60 * 60 * 24 * 7L;
        public static final String CONTEXT_KEY_SUFFIX = ".container.agent_id";

        public static String genAgentContextKey(String jobId) {
            return "jobs." + jobId + CONTEXT_KEY_SUFFIX;
        }

        public static String genAgentReuseMutexLockKey(String projectId, String agentId) {
            return "lock:agent:reuse:project:" + projectId + ":agent:" + agentId + ":lock";
        }

        public static String genAgentReuseMutexQueueKey(String projectId, String agentId) {
            return "lock:agent:reuse:project:" + projectId + ":agent:" + agentId + ":queue";
        }

        public static String genAgentReuseMutexLinkTipKey(String buildId) {
            return "agent:reuse:linkTip:" + buildId;
        }
    }

    public enum AgentReuseMutexType {
        AGENT_ID,
        AGENT_NAME,
        AGENT_ENV_ID,
        AGENT_ENV_NAME,
        AGENT_DEP_VAR;

        public boolean isEnvType() {
            return this == AGENT_ENV_ID || this == AGENT_ENV_NAME;
        }

        public boolean isAgentType() {
            return this == AGENT_ID || this == AGENT_NAME;
        }

        public boolean needEngineLock() {
            return isAgentType();
        }

        public AgentType toAgentType() {
            switch (this) {
                case AGENT_ID:
                case AGENT_ENV_ID:
                    return AgentType.ID;
                case AGENT_NAME:
                case AGENT_ENV_NAME:
                    return AgentType.NAME;
                default:
                    return null;
            }
        }
    }
}
